@@ -331,10 +331,10 @@ class BuildEnvironment:
         """
         if suffix:
             warnings.warn('The suffix argument for doc2path() is deprecated.',
-                          RemovedInSphinx40Warning)
+                          RemovedInSphinx40Warning, stacklevel=2)
         if base not in (True, False, None):
             warnings.warn('The string style base argument for doc2path() is deprecated.',
-                          RemovedInSphinx40Warning)
+                          RemovedInSphinx40Warning, stacklevel=2)
 
         pathname = self.project.doc2path(docname, base is True)
         if suffix:
@@ -387,13 +387,14 @@ class BuildEnvironment:
                 # add catalog mo file dependency
                 repo = CatalogRepository(self.srcdir, self.config.locale_dirs,
                                          self.config.language, self.config.source_encoding)
+                mo_paths = {c.domain: c.mo_path for c in repo.catalogs}
                 for docname in self.found_docs:
                     domain = docname_to_domain(docname, self.config.gettext_compact)
-                    for catalog in repo.catalogs:
-                        if catalog.domain == domain:
-                            self.dependencies[docname].add(catalog.mo_path)
+                    if domain in mo_paths:
+                        self.dependencies[docname].add(mo_paths[domain])
         except OSError as exc:
-            raise DocumentError(__('Failed to scan documents in %s: %r') % (self.srcdir, exc))
+            raise DocumentError(__('Failed to scan documents in %s: %r') %
+                                (self.srcdir, exc)) from exc
 
     def get_outdated_files(self, config_changed: bool) -> Tuple[Set[str], Set[str], Set[str]]:
         """Return (added, changed, removed) sets."""
@@ -511,8 +512,8 @@ class BuildEnvironment:
         """
         try:
             return self.domains[domainname]
-        except KeyError:
-            raise ExtensionError(__('Domain %r is not registered') % domainname)
+        except KeyError as exc:
+            raise ExtensionError(__('Domain %r is not registered') % domainname) from exc
 
     # --------- RESOLVING REFERENCES AND TOCTREES ------------------------------
 
@@ -593,7 +594,9 @@ class BuildEnvironment:
 
         def traverse_toctree(parent: str, docname: str) -> Iterator[Tuple[str, str]]:
             if parent == docname:
-                logger.warning(__('self referenced toctree found. Ignored.'), location=docname)
+                logger.warning(__('self referenced toctree found. Ignored.'),
+                               location=docname, type='toc',
+                               subtype='circular')
                 return
 
             # traverse toctree by pre-order
