@@ -1608,8 +1608,17 @@ class DataDocumenter(ModuleLevelDocumenter):
             # obtain annotation for this data
             try:
                 annotations = get_type_hints(self.parent)
+            except NameError:
+                # Failed to evaluate ForwardRef (maybe TYPE_CHECKING)
+                annotations = safe_getattr(self.parent, '__annotations__', {})
             except TypeError:
                 annotations = {}
+            except KeyError:
+                # a broken class found (refs: https://github.com/sphinx-doc/sphinx/issues/8084)
+                annotations = {}
+            except AttributeError:
+                # AttributeError is raised on 3.5.2 (fixed by 3.5.3)
+                annotations = {}
 
             if self.objpath[-1] in annotations:
                 objrepr = stringify_typehint(annotations.get(self.objpath[-1]))
@@ -1978,8 +1987,17 @@ class AttributeDocumenter(DocstringStripSignatureMixin, ClassLevelDocumenter):
             # obtain type annotation for this attribute
             try:
                 annotations = get_type_hints(self.parent)
+            except NameError:
+                # Failed to evaluate ForwardRef (maybe TYPE_CHECKING)
+                annotations = safe_getattr(self.parent, '__annotations__', {})
             except TypeError:
                 annotations = {}
+            except KeyError:
+                # a broken class found (refs: https://github.com/sphinx-doc/sphinx/issues/8084)
+                annotations = {}
+            except AttributeError:
+                # AttributeError is raised on 3.5.2 (fixed by 3.5.3)
+                annotations = {}
 
             if self.objpath[-1] in annotations:
                 objrepr = stringify_typehint(annotations.get(self.objpath[-1]))
