@@ -1,32 +1,29 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.extension
     ~~~~~~~~~~~~~~~~
 
     Utilities for Sphinx extensions.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from six import iteritems
+from typing import Any, Dict
 
+from sphinx.config import Config
 from sphinx.errors import VersionRequirementError
 from sphinx.locale import __
 from sphinx.util import logging
 
 if False:
     # For type annotation
-    from typing import Any, Dict  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-    from sphinx.config import Config  # NOQA
+    from sphinx.application import Sphinx
 
 logger = logging.getLogger(__name__)
 
 
-class Extension(object):
-    def __init__(self, name, module, **kwargs):
-        # type: (unicode, Any, Any) -> None
+class Extension:
+    def __init__(self, name: str, module: Any, **kwargs: Any) -> None:
         self.name = name
         self.module = module
         self.metadata = kwargs
@@ -43,13 +40,12 @@ class Extension(object):
         self.parallel_write_safe = kwargs.pop('parallel_write_safe', True)
 
 
-def verify_needs_extensions(app, config):
-    # type: (Sphinx, Config) -> None
+def verify_needs_extensions(app: "Sphinx", config: Config) -> None:
     """Verify the required Sphinx extensions are loaded."""
     if config.needs_extensions is None:
         return
 
-    for extname, reqversion in iteritems(config.needs_extensions):
+    for extname, reqversion in config.needs_extensions.items():
         extension = app.extensions.get(extname)
         if extension is None:
             logger.warning(__('The %s extension is required by needs_extensions settings, '
@@ -63,9 +59,8 @@ def verify_needs_extensions(app, config):
                                           (extname, reqversion, extension.version))
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
-    app.connect('config-inited', verify_needs_extensions)
+def setup(app: "Sphinx") -> Dict[str, Any]:
+    app.connect('config-inited', verify_needs_extensions, priority=800)
 
     return {
         'version': 'builtin',
